//
//  SOSISignature.m
//  SOSI digital signature POC
//
//  Created by Christian Gasser on 01/08/2012.
//  Copyright (c) 2012 Lakeside. All rights reserved.
//

// Utility class with methods for loading key-pairs and signing data

#import <CommonCrypto/CommonDigest.h>
#import "SOSISignature.h"
#import "SOSIBase64.h"


@implementation SOSISignature

+ (NSString *)hashAndSignBytes:(NSData *)bytes usingPKCS12File:(NSString *)path usingPassword:(NSString *)password outCertificate:(NSString **)certificate {

    NSData *pkcs12Data = [[NSData alloc] initWithContentsOfFile:path];
    CFDataRef pkcs12CFData = (__bridge CFDataRef) pkcs12Data;

    OSStatus status;
    SecIdentityRef identityRef;
    status = [SOSISignature extractIdentity:&identityRef usingPKCS12Data:pkcs12CFData usingPassword:password];
    NSLog(@"Identity status: %li", status);

    SecCertificateRef certificateRef = NULL;
    status = SecIdentityCopyCertificate (identityRef, &certificateRef);
    NSLog(@"Certificate status: %li", status);
    CFDataRef cfCertificateData = SecCertificateCopyData(certificateRef);
    *certificate = [SOSIBase64 encodeBase64WithData:(__bridge NSData *)cfCertificateData];

    SecKeyRef privateKeyRef;
    status = SecIdentityCopyPrivateKey(identityRef, &privateKeyRef);
    NSLog(@"PrivateKey status: %li", status);

    const uint8_t *dataToSign = [SOSISignature hashBytes:bytes];

    NSLog(@"SHA1: %@", [SOSIBase64 encodeBase64WithData:[[NSData alloc] initWithBytes:dataToSign length:CC_SHA1_DIGEST_LENGTH]]);

    size_t signedBytesSize = SecKeyGetBlockSize(privateKeyRef);

    // Malloc a buffer to hold signature.
    uint8_t *signedBytes = malloc( signedBytesSize * sizeof(uint8_t) );
    memset((void *)signedBytes, 0x0, signedBytesSize);

    status = SecKeyRawSign(privateKeyRef, kSecPaddingPKCS1SHA1, dataToSign, CC_SHA1_DIGEST_LENGTH, signedBytes, &signedBytesSize);

    NSLog(@"Signature status: %li", status);
    NSLog(@"Signature size: %li", signedBytesSize);

    NSData *signature = [NSData dataWithBytesNoCopy:signedBytes length:(NSUInteger)signedBytesSize freeWhenDone:YES];

    return [SOSIBase64 encodeBase64WithData:signature];
}

/* Extracts key-pair form pkcs12 data */
+ (OSStatus)extractIdentity:(SecIdentityRef *)outIdentity usingPKCS12Data:(CFDataRef)pkcs12Data usingPassword:(NSString *)inPassword {

    OSStatus securityError;
    CFStringRef password = (__bridge CFStringRef) inPassword;

    const void *keys[] = {kSecImportExportPassphrase};
    const void *values[] = {password};

    CFDictionaryRef optionsDictionary = CFDictionaryCreate(NULL, keys, values, 1, NULL, NULL);
    CFArrayRef items = CFArrayCreate(NULL, 0, 0, NULL);
    securityError = SecPKCS12Import(pkcs12Data, optionsDictionary, &items);

    if (securityError == errSecSuccess) {
        CFDictionaryRef identityAndTrust = CFArrayGetValueAtIndex(items, 0);
        const void *tempIdentity;
        tempIdentity = CFDictionaryGetValue(identityAndTrust, kSecImportItemIdentity);
        *outIdentity = (SecIdentityRef) tempIdentity;
    }

    if (optionsDictionary) {
        CFRelease(optionsDictionary);
    }

    return securityError;

}

+ (uint8_t *)hashBytes:(NSData *)bytes {

    CC_SHA1_CTX ctx;
    uint8_t * hashBytes;

    // Malloc a buffer to hold hash.
    hashBytes = malloc( CC_SHA1_DIGEST_LENGTH * sizeof(uint8_t) );
    memset((void *)hashBytes, 0x0, CC_SHA1_DIGEST_LENGTH);

    CC_SHA1_Init(&ctx);
    CC_SHA1_Update(&ctx, (void *)[bytes bytes], [bytes length]);
    CC_SHA1_Final(hashBytes, &ctx);

    return hashBytes;

}

@end
