//
//  Created by Christian Gasser on 02/08/2012.
//  Copyright (c) 2012 Lakeside. All rights reserved.
//


// String-based templates used to create idcards and SOSI-GW requests
// In a real-world application one should use proper XML instead

#import "SOSITemplates.h"


@implementation SOSITemplates

// placeholders: createdTimestamp, idCard, messageId, body
static NSString *envelope = @"<?xml version='1.0' encoding='UTF-8'?><soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\" xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:medcom=\"http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd\" xmlns:sosi=\"http://www.sosi.dk/sosi/2006/04/sosi-1.0.xsd\" xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:wst=\"http://schemas.xmlsoap.org/ws/2005/02/trust\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\" xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" id=\"Envelope\"><soapenv:Header><wsse:Security><wsu:Timestamp><wsu:Created>%@</wsu:Created></wsu:Timestamp>%@</wsse:Security><medcom:Header><medcom:SecurityLevel>1</medcom:SecurityLevel><medcom:Linking><medcom:MessageID>%@</medcom:MessageID></medcom:Linking><medcom:RequireNonRepudiationReceipt>no</medcom:RequireNonRepudiationReceipt></medcom:Header></soapenv:Header><soapenv:Body>%@</soapenv:Body></soapenv:Envelope>";

// placeholders: issueInstant, cpr(NameID), NotBefore, NotOnOrAfter, idCardID, cpr, cvr
static NSString *idCard = @"<saml:Assertion IssueInstant=\"%@\" Version=\"2.0\" id=\"IDCard\"><saml:Issuer>SOSI iOS PoC</saml:Issuer><saml:Subject><saml:NameID Format=\"medcom:cprnumber\">%@</saml:NameID></saml:Subject><saml:Conditions NotBefore=\"%@\" NotOnOrAfter=\"%@\" /><saml:AttributeStatement id=\"IDCardData\"><saml:Attribute Name=\"sosi:IDCardID\"><saml:AttributeValue>%@</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"sosi:IDCardVersion\"><saml:AttributeValue>1.0.1</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"sosi:IDCardType\"><saml:AttributeValue>user</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"sosi:AuthenticationLevel\"><saml:AttributeValue>1</saml:AttributeValue></saml:Attribute></saml:AttributeStatement><saml:AttributeStatement id=\"UserLog\"><saml:Attribute Name=\"medcom:UserCivilRegistrationNumber\"><saml:AttributeValue>%@</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserGivenName\"><saml:AttributeValue>Hans</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserSurName\"><saml:AttributeValue>Hansen</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserEmailAddress\"><saml:AttributeValue>hans@hans.dk</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserRole\"><saml:AttributeValue>doktor</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserAuthorizationCode\"><saml:AttributeValue>DM301</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:UserOccupation\"><saml:AttributeValue>maskinarbejder</saml:AttributeValue></saml:Attribute></saml:AttributeStatement><saml:AttributeStatement id=\"SystemLog\"><saml:Attribute Name=\"medcom:ITSystemName\"><saml:AttributeValue>SOSITEST</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:CareProviderID\" NameFormat=\"medcom:cvrnumber\"><saml:AttributeValue>%@</saml:AttributeValue></saml:Attribute><saml:Attribute Name=\"medcom:CareProviderName\"><saml:AttributeValue>Lægehuset's Støtteforening \"Jakob\" af &lt;1913> kreds 'A'</saml:AttributeValue></saml:Attribute></saml:AttributeStatement></saml:Assertion>";

static const NSString *requestDigestRequestBody = @"<sosigw:requestIdCardDigestForSigningRequestBody xmlns:sosigw=\"http://sosi.dk/gw/2007.09.01\">whatever</sosigw:requestIdCardDigestForSigningRequestBody>";

//placeholders: signature, certificate (both in Base64)
static NSString *signIdCardRequestBody = @"<sosigw:signIdCardRequestBody xmlns:sosigw=\"http://sosi.dk/gw/2007.09.01\"><ds:SignatureValue>%@</ds:SignatureValue><ds:KeyInfo><ds:X509Data><ds:X509Certificate>%@</ds:X509Certificate></ds:X509Data></ds:KeyInfo></sosigw:signIdCardRequestBody>";

static const NSString *logoutRequestBody = @"<sosigw:logoutRequestBody xmlns:sosigw=\"http://sosi.dk/gw/2007.09.01\">whatever</sosigw:logoutRequestBody>";


+ (NSString *)createIdCardWithCPR:(NSString *)cpr andCVR:(NSString *)cvr {

    NSDate *now = [NSDate date];
    NSDate *nowPlus24Hours = [now dateByAddingTimeInterval:60 * 60 * 24];

    return [NSString stringWithFormat:idCard, [self formatDate:now], cpr, [self formatDate:now], [self formatDate:nowPlus24Hours], [self generateUuidString], cpr, cvr];
}

+ (NSString *)createRequestForDigestRequestWithIdCard:(NSString *)idCard {
    return [NSString stringWithFormat:envelope, [self formatDate:[NSDate date]], idCard, [self generateUuidString], requestDigestRequestBody];
}

+ (NSString *)createSignIdCardRequestWithIdCard:(NSString *)idCard andSignature:(NSString *)signature andCertificate:(NSString *)certificate {
    NSString *body = [NSString stringWithFormat:signIdCardRequestBody, signature, certificate];
    return [NSString stringWithFormat:envelope, [self formatDate:[NSDate date]], idCard, [self generateUuidString], body];
}

+ (NSString *)createLogoutRequestWithIdCard:(NSString *)idCard {
    return [NSString stringWithFormat:envelope, [self formatDate:[NSDate date]], idCard, [self generateUuidString], logoutRequestBody];
}

+ (NSString *)formatDate:(NSDate *)date {
    static NSDateFormatter *sRFC3339DateFormatter = nil;

    if (sRFC3339DateFormatter == nil) {
        sRFC3339DateFormatter = [[NSDateFormatter alloc] init];
        NSLocale *enUSPOSIXLocale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];

        [sRFC3339DateFormatter setLocale:enUSPOSIXLocale];
        [sRFC3339DateFormatter setDateFormat:@"yyyy'-'MM'-'dd'T'HH':'mm':'ss'Z'"];
        [sRFC3339DateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
    }

    return [sRFC3339DateFormatter stringFromDate:date];

}

+ (NSString *)generateUuidString {
    // create a new UUID which you own
    CFUUIDRef uuid = CFUUIDCreate(kCFAllocatorDefault);

    // create a new CFStringRef (toll-free bridged to NSString)
    // that you own
    NSString *uuidString = (__bridge NSString *) CFUUIDCreateString(kCFAllocatorDefault, uuid);

    // release the UUID
    CFRelease(uuid);

    return uuidString;
}

@end