//
//  SOSIViewController.m
//  SOSI digital signature POC
//
//  Created by Christian Gasser on 03/07/2012.
//  Copyright (c) 2012 Lakeside. All rights reserved.

// The application's main controller

#import "SOSIViewController.h"
#import "SOSISignature.h"
#import "SOSITemplates.h"
#import "SOSISoapClient.h"
#import "SOSIBase64.h"

@interface SOSIViewController ()

@end

@implementation SOSIViewController {
    NSString *_idCard;
    SOSISoapClient *_soapClient;
    NSString *_certificateFileName;
    NSString *_certificateFileSuffix;
    NSString *_password;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (IBAction)goDo:(id)sender {
    NSLog(@"Go do!");

    NSString *sosiGatewayServiceEndpoint = @"http://localhost:8080/sosigw/service/sosigw";

    /* MOCES1 test-certificate */
    _certificateFileName = @"MikeValidMOCES";
    _certificateFileSuffix = @"pkcs12";
    _password = @"!234Qwer";
    NSString *cpr = @"1111111118";
    NSString *cvr = @"19343634";

/*    Uncomment to test with a MOCES2 test-certificate instead */
//    /* MOCES1 test-certificate */
//    _certificateFileName = @"tc01-valid-moces-cpr1";
//    _certificateFileSuffix = @"pkcs12";
//    _password = @"Test1234";
//    NSString *cpr = @"1602763141";
//    NSString *cvr = @"26912865";

    _idCard = [SOSITemplates createIdCardWithCPR:cpr andCVR:cvr];

    NSString *requestForDigestRequest = [SOSITemplates createRequestForDigestRequestWithIdCard:_idCard];

    _soapClient = [SOSISoapClient createWithEndpoint:sosiGatewayServiceEndpoint];

    /* callback method to be called upon completion */
    void (^callback) (NSString *) = ^(NSString *response) {[self handleDigestResponse:response];};
    [_soapClient postRequest:requestForDigestRequest withSoapAction:@"http://sosi.dk/gw/2007.09.01#requestIdCardDigestForSigning" andCallback:callback ];

}

- (void) handleDigestResponse:(NSString *) response {
    NSString *digest = [self extractDigest:response];
    NSLog(@"Digest to sign: %@", digest);
    NSData *bytesToSign = [SOSIBase64 decodeBase64WithString:digest];

    NSString *path = [[NSBundle mainBundle] pathForResource:_certificateFileName ofType:_certificateFileSuffix];

/*    Sample code that shows how to load a certificate that has a user has imported through iTunes file sharing*/
//    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
//    NSString *documentsDir = [paths objectAtIndex:0];
//
//    NSError *error;
//    NSArray *directoryContents = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:documentsDir error:&error];
//
//    if (error) {
//        NSLog(@"Error reading from Documents dir: %@", error);
//    }
//
//    for (int i = 0; i < directoryContents.count; i++) {
//        NSLog(@"File in Documents dir: %@", [directoryContents objectAtIndex:i]);
//    }
//
//    NSString *path = [[NSBundle bundleWithPath:documentsDir] pathForResource:@"<chosen certificate file name>" ofType:@"<chosen certificate file extension>"];

    NSString *certificateBase64;
    NSString *signatureBase64 = [SOSISignature hashAndSignBytes:bytesToSign usingPKCS12File:path usingPassword:_password outCertificate:&certificateBase64];

    NSLog(@"Certificate: %@", certificateBase64);
    NSLog(@"Signature: %@", signatureBase64);

    NSString *signIdCardRequest = [SOSITemplates createSignIdCardRequestWithIdCard:_idCard andSignature:signatureBase64 andCertificate:certificateBase64];

    void (^callback) (NSString *) = ^(NSString *response) {[self handleSignResponse:response];};
    [_soapClient postRequest:signIdCardRequest withSoapAction:@"http://sosi.dk/gw/2007.09.01#signIdCard" andCallback:callback ];

}

- (void)handleSignResponse:(NSString *)response {
    NSString *logoutRequest = [SOSITemplates createLogoutRequestWithIdCard:_idCard];

    void (^callback) (NSString *) = ^(NSString *response) {[self handleLogoutResponse:response];};
    [_soapClient postRequest:logoutRequest withSoapAction:@"http://sosi.dk/gw/2007.09.01#logout" andCallback:callback ];

}

- (void)handleLogoutResponse:(NSString *)response {
    NSLog(@"All done!");
}


- (NSString *)extractDigest:(NSString *)response {
    NSRange startRange = [response rangeOfString:@"DigestValue>"];
    NSString *temp = [response substringFromIndex:startRange.location + startRange.length];
    NSRange stopRange = [temp rangeOfString:@"<"];
    return [temp substringToIndex:stopRange.location];
}

@end
